# -*- coding: utf-8 -*-
""" Viewing components for WAeUP objects.
"""
import grok
from waeup.browser import WAeUPPage, WAeUPEditFormPage, WAeUPAddFormPage
from waeup.interfaces import (IUniversity, IWAeUPObject, IUserContainer,
                              IUserAccount, IFacultyContainer)
from waeup.widgets.interfaces import ITableProvider
from zope.app.security.interfaces import (IAuthentication,
                                          IUnauthenticatedPrincipal,
                                          ILogout)
from zope.component import getUtility
from zope.exceptions import DuplicationError

grok.templatedir('templates')

#
# Login/logout pages...
#

class LoginPage(WAeUPPage):
    """A login page, available for all objects.
    """
    grok.name('login')
    grok.context(IWAeUPObject)

    title = u'Login'
    camefrom = None

    def update(self, SUBMIT=None, camefrom=None):
        self.camefrom = camefrom
        if SUBMIT is not None:
            if self.request.principal.id != 'zope.anybody':
                self.flash('You logged in.')
                if not self.camefrom:
                    # User might have entered the URL directly. Let's beam
                    # him back to our context.
                    self.redirect(self.url(self.context))
                    return
                self.redirect(self.camefrom)
                return
            self.flash('You entered wrong credentials!')


class LoginStaffPage(LoginPage):
    """A login page for staff members.
    """
    grok.name('loginstaff')
    grok.template('loginstaffpage')
    title = u'Staff Login'


class LoginStudentPage(LoginPage):
    """A login page for students.
    """
    grok.name('loginstudent')
    grok.template('loginstudentpage')
    title = u'Student Login'

class LogoutPage(WAeUPPage):
    """A logout page. Calling this page will log the current user out.
    """
    grok.context(IWAeUPObject)
    grok.name('logout')
    
    def update(self):
        if not IUnauthenticatedPrincipal.providedBy(self.request.principal):
            auth = getUtility(IAuthentication)
            ILogout(auth).logout(self.request)
            self.flash("You have been logged out. Thanks for using WAeUP SRP!")
        self.redirect(self.url(self.context))


#
# University related pages...
#

class UniversityPage(WAeUPPage):
    """ The main university page.
    """
    grok.name('index')
    grok.context(IUniversity)
    title = u'Welcome to SRP'
    

class Manage(WAeUPEditFormPage):
    """Manage the basic properties of a `University` instance.
    """
    form_fields = grok.AutoFields(IUniversity)
    grok.context(IUniversity)
    grok.require('waeup.manageUniversity')
    title = u'Edit settings'

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        self.flash('Settings have been saved.')
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.redirect(self.url(self.context))
        self.flash('Settings have been saved.')
        return

    @grok.action('Cancel')
    def cancel(self, **data):
        self.flash('Action cancelled.')
        self.redirect(self.url(self.context))
        return

#
# User container pages...
#
    
class UserContainerPage(WAeUPPage):
    """Overview page for all local users.
    """
    grok.require('waeup.manageUsers')
    grok.context(IUserContainer)
    grok.name('index')
    title = 'Users'
    
    def update(self, userid=None, adduser=None, edit=None, delete=None):
        if edit is not None and userid is not None:
            self.redirect(self.url(userid))
        if delete is not None and userid is not None:
            self.context.delUser(userid)
            self.flash('User %s successfully deleted.' % userid)

class AddUserFormPage(WAeUPAddFormPage):
    grok.require('waeup.manageUsers')
    grok.context(IUserContainer)
    grok.name('add')
    form_fields = grok.AutoFields(IUserAccount)
    label = 'Add user'
    title = 'Add user'

    @grok.action('Add user')
    def addUser(self, **data):
        name = data['name']
        title = data['title']
        description = data['description']
        password = data['password']
        roles = data['roles']
        try:
            self.context.addUser(name, password, title=title,
                                 description=description, roles=roles)
        except DuplicationError:
            self.status = Invalid('The userid chosen already exists '
                                  'in the database')
            return
        self.redirect(self.url(self.context))

class UserEditFormPage(WAeUPEditFormPage):
    """Edit a user account.
    """
    grok.context(IUserAccount)
    grok.name('index')
    
    form_fields = grok.AutoFields(IUserAccount)
    grok.require('waeup.manageUsers')

    title = "Edit user"

    @grok.action('Save')
    def save(self, **data):
        self.applyData(self.context, **data)
        self.flash('User settings have been saved.')
        return
    
    @grok.action('Save and return')
    def saveAndReturn(self, **data):
        self.applyData(self.context, **data)
        self.flash('User settings have been saved.')
        self.redirect(self.url(self.context.__parent__))
        return

    @grok.action('Cancel')
    def cancel(self, **data):
        self.redirect(self.url(self.context.__parent__))
        return


#
# FacultyContainer pages...
#

class FacultyContainerPage(WAeUPPage):
    grok.context(IFacultyContainer)
    grok.name('index')
    title = 'Faculties'

    def update(self):
        self.table = ITableProvider(self.context).getTables(self)[0]
        self.table.need()
    
    def getFaculties(self):
        """Convinience method to create a sorted list of faculties.

        It provides a list of dicts with entries for all data needed by
        usual list templates.
        """
        result = []
        for key, val in self.context.items():
            result.append(dict(id=key, name=val.title))
        return result
